import sys, subprocess, platform
from termcolor import colored

is_windows = sys.platform == 'win32'
is_linux   = sys.platform.startswith('linux')
is_macos   = sys.platform == 'darwin'
if not (is_windows or is_linux or is_macos):
    raise RuntimeError(f'MamaBuild unsupported platform {sys.platform}')

machine = platform.machine()
is_aarch64 = machine == 'aarch64' or machine == 'arm64'
is_x86_64 = machine == 'x86_64' or machine == 'AMD64'
is_x86 = machine == 'x86' or machine == 'i386'

class System:
    windows = is_windows
    linux   = is_linux
    macos   = is_macos
    aarch64 = is_aarch64
    x86_64  = is_x86_64
    x86     = is_x86

# Available text colors:
#     black, red, green, yellow, blue, magenta, cyan, white,
#     light_grey, dark_grey, light_red, light_green, light_yellow, light_blue,
#     light_magenta, light_cyan.

# Available text highlights:
#     on_black, on_red, on_green, on_yellow, on_blue, on_magenta, on_cyan, on_white,
#     on_light_grey, on_dark_grey, on_light_red, on_light_green, on_light_yellow,
#     on_light_blue, on_light_magenta, on_light_cyan.
class Color:
    DEFAULT = None
    RED = "red"
    GREEN = "green"
    YELLOW = "yellow"
    BLUE = "blue"
    MAGENTA = "magenta"


# on windows use colorama to enable ANSI color escape sequences
if System.windows:
    from colorama import just_fix_windows_console
    just_fix_windows_console()


def get_colored_text(text:str, color):
    return colored(text, color=color) if color else text


def console(text:str, color=None, end="\n"):
    """ Always flush to support most build environments """
    print(get_colored_text(text, color), end=end, flush=True)


def error(text:str):
    """ Prints a message as an error, usually colored red """
    console(text, color=Color.RED)




from __future__ import annotations
from typing import TYPE_CHECKING
from typing import Callable
from .generic_yocto import GenericYocto


if TYPE_CHECKING:
    from ..build_config import BuildConfig


class Imx8mp(GenericYocto):
    BUILD_DIR = 'imx8mp' # Constant: i.MX8M Plus build dir name

    def __init__(self, config: BuildConfig):
        # NXP i.MX8M Plus (imx8mp) is a Cortex-A53 based SoC with integrated NPU, 
        # and is supported by Yocto SDKs provided by NXP and 3rd parties like IMD Tec
        super().__init__(Imx8mp.BUILD_DIR, Imx8mp.BUILD_DIR.upper(), config)
        self.build_dir = Imx8mp.BUILD_DIR  # override generic build dir
        self.host_name = 'aarch64-poky-linux' # override generic host name


    def init_toolchain(self, toolchain_dir=None, toolchain_file=None):
        paths = []
        if toolchain_dir: paths += [ toolchain_dir ]
        paths += [ '/opt/imdt-imx-xwayland/5.0.4' ]
        paths += [ '/opt/imx8mp-sdk' ]
        paths += [ 'imx8mp-toolchain' ]

        self.distro_version = (5, 0, 4) # default distro version for i.MX8MP builds, overridden by config

        # /opt/imdt-imx-xwayland/5.0.4/sysroots/x86_64-pokysdk-linux/usr/bin/aarch64-poky-linux/aarch64-poky-linux-gcc
        compiler = 'usr/bin/aarch64-poky-linux/aarch64-poky-linux-gcc'
        default_toolchain = 'sysroots/x86_64-pokysdk-linux/usr/share/cmake/cortexa53-crypto-poky-linux-toolchain.cmake'

        self._yocto_toolchain_init(toolchain_dir, toolchain_file,
                                   paths=paths,
                                   envs=['IMX8MP_SDK_HOME'],
                                   compiler_name=compiler,
                                   sdk_name='x86_64-pokysdk-linux',
                                   sysroot_name='cortexa53-crypto-poky-linux',
                                   default_toolchain=default_toolchain) 


    def get_cxx_flags(self, add_flag: Callable[[str,str], None]):
        add_flag('-march', 'armv8-a')
        add_flag('-mcpu', 'cortex-a53+crypto')
        add_flag('-mlittle-endian')
        self._add_common_cxx_flags(add_flag)




import os
import shutil
import subprocess
import sys
from typing import Iterable, Optional

import mama
import pytest

def init(caller_file: str = '', clean_dirs: Optional[Iterable[str]] = None):
    # Needed for mama commands to perform work in the correct directory
    if caller_file:
        os.chdir(os.path.dirname(os.path.abspath(caller_file)))

    if clean_dirs is None:
        clean_dirs = ()

    for d in clean_dirs:
        rmdir(d)

def shell_exec(cmd: str, exit_on_fail: bool = True, echo: bool = True) -> int:
    if echo: print(f'exec: {cmd}')
    result = subprocess.run(cmd, shell=True)
    if result.returncode != 0 and exit_on_fail:
        pytest.fail(f'exec failed: code: {result.returncode} {cmd}')
    return result.returncode

def mama_exec(args: list[str], exit_on_fail: bool = True, echo: bool = True) -> int:
    """Calls mama.mamabuild() directly instead of shelling out to the mama CLI."""
    if echo: print(f'mama: {" ".join(args)}')
    try:
        mama.mamabuild(args, source_dir=os.getcwd())
        return 0
    except SystemExit as e:
        code = e.code if isinstance(e.code, int) else 1
        if code != 0 and exit_on_fail:
            pytest.fail(f'mama failed: code: {code} args: {args}')
        return code
    except Exception as e:
        if exit_on_fail:
            pytest.fail(f'mama failed: {e} args: {args}')
        return 1

def file_contains(filepath: str, text: str) -> bool:
    with open(filepath, 'r') as f:
        content = f.read()
    return text in content

def file_exists(filepath: str) -> bool:
    return os.path.isfile(filepath)

def is_windows() -> bool:
    return os.name == 'nt'

def is_linux() -> bool:
    return os.name == 'posix' and sys.platform != 'darwin'

def is_macos() -> bool:
    return sys.platform == 'darwin'

def executable_extension() -> str:
    if is_windows():
        return '.exe'

    return ''

def static_library_extension() -> str:
    if is_windows():
        return '.lib'
    else:
        return '.a'

def dynamic_library_extension() -> str:
    if is_windows():
        return '.dll'
    elif is_macos():
        return '.dylib'
    else:
        return '.so'

# Excludes for example android
def native_platform_name() -> str:
    if is_windows():
        return 'windows'
    elif is_linux():
        return 'linux'
    elif is_macos():
        return 'macos'
    else:
        raise Exception("Unsupported platform")

def onerror(func, path, _):
    import stat
    if not os.access(path, os.W_OK):
        os.chmod(path, stat.S_IWUSR)
        func(path)

def rmdir(path: str):
    if os.path.exists(path):
        shutil.rmtree(path, onerror=onerror)


from __future__ import annotations
from typing import TYPE_CHECKING
from typing import Callable
from .generic_yocto import GenericYocto


if TYPE_CHECKING:
    from ..build_config import BuildConfig


class Oclea(GenericYocto):
    BUILD_DIR = 'oclea' # Constant: Oclea Ambarella CV25 build dir name

    def __init__(self, config: BuildConfig):
        ## Ambarella CV25 by Oclea is a Cortex-A53 based SoC with a HW Encoder.
        super().__init__(Oclea.BUILD_DIR, Oclea.BUILD_DIR.upper(), config)
        self.build_dir = Oclea.BUILD_DIR  # override generic build dir
        self.host_name = 'aarch64-oclea-linux' # override generic host name


    def init_toolchain(self, toolchain_dir=None, toolchain_file=None):
        paths = []
        if toolchain_dir: paths += [ toolchain_dir ]
        # this is the primary search path for Linux cross-builds:
        paths += [ '/opt/oclea/1.0' ]
        # these are generic ones:
        paths += [ 'oclea-toolchain', 'oclea-toolchain/toolchain' ]

        self._yocto_toolchain_init(toolchain_dir, toolchain_file,
                                   paths=paths,
                                   envs=['OCLEA_HOME', 'OCLEA_SDK'],
                                   compiler_name='usr/bin/aarch64-oclea-linux/aarch64-oclea-linux-gcc',
                                   sdk_name='x86_64-ocleasdk-linux',
                                   sysroot_name='cortexa53-oclea-linux',
                                   default_toolchain='aarch64_oclea_toolchain.cmake')


    def get_cxx_flags(self, add_flag: Callable[[str,str], None]):
        add_flag('-march', 'armv8-a')
        add_flag('-mcpu', 'cortex-a53+crypto')
        add_flag('-mlittle-endian')
        self._add_common_cxx_flags(add_flag)

